@@ -1,7 +1,4 @@
-"""Requirements specific to SQLAlchemy's own unit tests.
-
-
-"""
+"""Requirements specific to SQLAlchemy's own unit tests."""
 
 from sqlalchemy import exc
 from sqlalchemy.sql import sqltypes
@@ -162,6 +159,10 @@ class DefaultRequirements(SuiteRequirements):
     def fk_constraint_option_reflection_onupdate_restrict(self):
         return only_on(["postgresql", "sqlite", self._mysql_80])
 
+    @property
+    def temp_table_comment_reflection(self):
+        return only_on(["postgresql", "mysql", "mariadb", "oracle"])
+
     @property
     def comment_reflection(self):
         return only_on(["postgresql", "mysql", "mariadb", "oracle", "mssql"])
@@ -212,6 +213,19 @@ class DefaultRequirements(SuiteRequirements):
             ]
         )
 
+    @property
+    def server_defaults(self):
+        """Target backend supports server side defaults for columns"""
+
+        return exclusions.open()
+
+    @property
+    def expression_server_defaults(self):
+        return skip_if(
+            lambda config: against(config, "mysql", "mariadb")
+            and not self._mysql_expression_defaults(config)
+        )
+
     @property
     def qmark_paramstyle(self):
         return only_on(["sqlite", "+pyodbc"])
@@ -301,7 +315,9 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def binary_literals(self):
         """target backend supports simple binary literals, e.g. an
-        expression like::
+        expression like:
+
+        .. sourcecode:: sql
 
             SELECT CAST('foo' AS BINARY)
 
@@ -431,6 +447,14 @@ class DefaultRequirements(SuiteRequirements):
             in self.get_isolation_levels(config)["supported"]
         )
 
+    @property
+    def skip_autocommit_rollback(self):
+        return exclusions.skip_if(
+            ["mssql+pymssql"],
+            "DBAPI has no means of testing the autocommit status of a "
+            "connection",
+        )
+
     @property
     def row_triggers(self):
         """Target must support standard statement-running EACH ROW triggers."""
@@ -491,6 +515,13 @@ class DefaultRequirements(SuiteRequirements):
             "Backend does not support UPDATE..FROM",
         )
 
+    @property
+    def update_from_returning(self):
+        """Target must support UPDATE..FROM syntax where RETURNING can
+        return columns from the non-primary FROM clause"""
+
+        return self.update_returning + self.update_from + skip_if("sqlite")
+
     @property
     def update_from_using_alias(self):
         """Target must support UPDATE..FROM syntax against an alias"""
@@ -522,7 +553,9 @@ class DefaultRequirements(SuiteRequirements):
         present in a subquery in the WHERE clause.
 
         This is an ANSI-standard syntax that apparently MySQL can't handle,
-        such as::
+        such as:
+
+        .. sourcecode:: sql
 
             UPDATE documents SET flag=1 WHERE documents.title IN
                 (SELECT max(documents.title) AS title
@@ -613,6 +646,16 @@ class DefaultRequirements(SuiteRequirements):
             + skip_if("oracle")
         )
 
+    @property
+    def inline_check_constraint_reflection(self):
+        return only_on(
+            [
+                "postgresql",
+                "sqlite",
+                "oracle",
+            ]
+        )
+
     @property
     def check_constraint_reflection(self):
         return only_on(
@@ -705,6 +748,24 @@ class DefaultRequirements(SuiteRequirements):
             ]
         )
 
+    @property
+    def ctes_with_values(self):
+        return only_on(
+            [
+                lambda config: against(config, "mysql")
+                and (
+                    (
+                        config.db.dialect._is_mariadb
+                        and config.db.dialect._mariadb_normalized_version_info
+                        >= (10, 2)
+                    )
+                ),
+                "mariadb>10.2",
+                "postgresql",
+                "sqlite>=3.8.3",
+            ]
+        )
+
     @property
     def ctes_with_update_delete(self):
         """target database supports CTES that ride on top of a normal UPDATE
@@ -962,6 +1023,10 @@ class DefaultRequirements(SuiteRequirements):
         """
         return exclusions.open()
 
+    @property
+    def nvarchar_types(self):
+        return only_on(["mssql", "oracle", "sqlite", "mysql", "mariadb"])
+
     @property
     def unicode_data_no_special_types(self):
         """Target database/dialect can receive / deliver / compare data with
@@ -991,7 +1056,12 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def arraysize(self):
-        return skip_if("+pymssql", "DBAPI is missing this attribute")
+        return skip_if(
+            [
+                no_support("+pymssql", "DBAPI is missing this attribute"),
+                no_support("+mysqlconnector", "DBAPI ignores this attribute"),
+            ]
+        )
 
     @property
     def emulated_lastrowid(self):
@@ -1462,9 +1532,7 @@ class DefaultRequirements(SuiteRequirements):
 
             expr = decimal.Decimal("15.7563")
 
-            value = e.scalar(
-                select(literal(expr))
-            )
+            value = e.scalar(select(literal(expr)))
 
             assert value == expr
 
@@ -1568,6 +1636,16 @@ class DefaultRequirements(SuiteRequirements):
     def postgresql_jsonb(self):
         return only_on("postgresql >= 9.4")
 
+    @property
+    def postgresql_working_nullable_domains(self):
+        # see https://www.postgresql.org/message-id/flat/a90f53c4-56f3-4b07-aefc-49afdc67dba6%40app.fastmail.com  # noqa: E501
+        return skip_if(
+            lambda config: (17, 0)
+            < config.db.dialect.server_version_info
+            < (17, 3),
+            "reflection of nullable domains broken on PG 17.0-17.2",
+        )
+
     @property
     def native_hstore(self):
         return self.any_psycopg_compatibility
@@ -1784,6 +1862,15 @@ class DefaultRequirements(SuiteRequirements):
         # 2. they dont enforce check constraints
         return not self._mysql_check_constraints_exist(config)
 
+    def _mysql_expression_defaults(self, config):
+        return (against(config, ["mysql", "mariadb"])) and (
+            config.db.dialect._support_default_function
+        )
+
+    @property
+    def mysql_expression_defaults(self):
+        return only_if(self._mysql_expression_defaults)
+
     def _mysql_not_mariadb_102(self, config):
         return (against(config, ["mysql", "mariadb"])) and (
             not config.db.dialect._is_mariadb
